import 'dart:async';

import 'package:flutter/services.dart';
import 'package:witsystem_device_plugin/MethodName.dart';
import 'package:witsystem_device_plugin/WitsystemSDK.dart';
import 'package:witsystem_device_plugin/code/DeviceOtherFunction.dart';
import 'package:witsystem_device_plugin/command/CardType.dart';
import 'package:witsystem_device_plugin/command/DeviceModel.dart';
import 'package:witsystem_device_plugin/command/FaceType.dart';
import 'package:witsystem_device_plugin/command/MarkType.dart';
import 'package:witsystem_device_plugin/command/PasswordType.dart';
import 'package:witsystem_device_plugin/code/DeviceFunctionType.dart';
import 'package:witsystem_device_plugin/enity/DeviceSoftwareInfo.dart';
import 'package:witsystem_device_plugin/enity/StateCode.dart';

import 'dart:convert' as convert;

///设备管理

class DeviceManage extends WisystemSDk {
  final List<StreamSubscription> _subscriptionList = [];

  final Stream<dynamic> _deviceChangeEvent =
      const EventChannel(ChannelName.deviceChage)
          .receiveBroadcastStream()
          .map((event) => convert.json.decode(event.toString()));

  factory DeviceManage() => _getInstance();

  static DeviceManage get instance => _getInstance();
  static DeviceManage? _instance;

  DeviceManage._internal();

  static DeviceManage _getInstance() {
    _instance ??= DeviceManage._internal();
    return _instance!;
  }

  /// 获得设备列表
  Future<List> deviceList() async {
    try {
      String json = await methodChannel.invokeMethod("deviceList");
      return Future.value(convert.json.decode(json));
    } catch (e) {
      rethrow;
    }
  }

  /// 获得指定设备的信息
  Future<Map<String, dynamic>?> device(String deviceId) async {
    try {
      String json =
          await methodChannel.invokeMethod("device", {"deviceId": deviceId});
      return Future.value(convert.json.decode(json));
    } catch (e) {
      rethrow;
    }
  }

  /// 刷新指定设备的数据
  Future<bool> refurbishDevice(String deviceId) async {
    return await methodChannel
        .invokeMethod("refurbishDevice", {"deviceId": deviceId});
  }

  /// 刷新所有设备的数据
  Future<bool> refurbishAllDevice() async {
    return await methodChannel.invokeMethod("refurbishAllDevice");
  }

  /// 获得设备的状态信息
  Future<StateCode?> getDeviceStateInfo(String deviceId) async {
    String info = await methodChannel
        .invokeMethod("getDeviceStateInfo", {"deviceId": deviceId});
    if (info == "") return Future.value();
    var decode = convert.json.decode(info);
    return Future.value(StateCode(
      battery: decode["battery"],
      volume: decode["volume"],
      mode: decode["mode"],
      temp: decode["temp"],
      lockTongue: decode["lockTongue"],
      mainLockTongue: decode["mainLockTongue"],
      door: decode["door"],
      antiLockButton: decode["antiLockButton"],
      keyCylinder: decode["keyCylinder"],
      card: decode["card"],
    ));
  }

  /// 获得设备软件的支持信息
  Future<DeviceSoftwareInfo?> getDeviceSoftwareFunction(String deviceId) async {
    String info = await methodChannel
        .invokeMethod("getDeviceSoftwareFunction", {"deviceId": deviceId});
    if (info == "") return Future.value();
    var decode = convert.json.decode(info);
    print("flutter《《《《《$decode");
    return Future.value(DeviceSoftwareInfo(
      deviceType: decode["deviceType"],
      passwordSum: decode["passwordSum"],
      //先吧枚举类型转成集合由于转换的集合不支持修改，所以把转换好的集合添加代新集合然后在刷选出包含枚举每次的数据，下面的几个类型都是这么实现的
      passwordType: [...PasswordType.values]..retainWhere(
          (element) => decode["passwordType"].contains((element).name)),
      cardSum: decode["cardSum"],
      cardType: [...CardType.values]
        ..retainWhere((element) => decode["cardType"].contains((element).name)),
      markSum: decode["markSum"],
      markType: [...MarkType.values]
        ..retainWhere((element) => decode["markType"].contains((element).name)),
      faceSum: decode["faceSum"],
      faceType: [...FaceType.values]
        ..retainWhere((element) => decode["faceType"].contains((element).name)),
      modelType: [
        ...DeviceModel.values
      ]..retainWhere((element) => decode["modelType"].contains((element).name)),
      connectionNumber: decode["connectionNumber"],
      otherFunction: [...DeviceOtherFunction.values]..retainWhere(
          (element) => decode["otherFunction"].contains((element).name)),
    ));
  }

  ///查询指定设备是否  支持指定的功能或者是硬件
  Future<bool> isDeviceFunction(
      String deviceId, DeviceFunctionType type) async {
    try {


      return await methodChannel.invokeMethod(
          "isDeviceFunction", {"deviceId": deviceId, "type": type.name});
    } catch (e) {
      rethrow;
    }
  }

  ///查询指定设备的用户列表
  Future<dynamic> deviceUserList(String deviceId) async {
    try {
      String str = await methodChannel
          .invokeMethod("deviceUserList", {"deviceId": deviceId});
      var decode = convert.json.decode(str);
      return convert.json.decode(decode["data"]);
    } catch (e) {
      rethrow;
    }
  }

  ///冻结或者解冻用户
  Future<bool> freezeOrUnfreeze(
      String deviceId, String freezeUserUuid, bool isFreeze) async {
    try {
      await methodChannel.invokeMethod("freezeOrUnfreeze", {
        "deviceId": deviceId,
        "freezeUserUuid": freezeUserUuid,
        "isFreeze": isFreeze
      });
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }

  ///管理员或者高等级用户，删除用户
  Future<bool> manageDeleteDeviceUser(
      String deviceId, String deleteUserUuid) async {
    try {
      await methodChannel.invokeMethod("manageDeleteDeviceUser",
          {"deviceId": deviceId, "deleteUserUuid": deleteUserUuid});
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }

  ///发送设备
  Future<bool> sendDevice(String userId, String deviceId, String startDate,
      String endDate, String authType, String repeatType) async {
    try {
      await methodChannel.invokeMethod("sendDevice", {
        "userId": userId,
        "deviceId": deviceId,
        "startDate": startDate,
        "endDate": endDate,
        "authType": authType,
        "repeatType": repeatType,
      });
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }

  ///转让设备
  Future<bool> translationDevice(String userId, String deviceId) async {
    try {
      await methodChannel.invokeMethod(
          "translationDevice", {"userId": userId, "deviceId": deviceId});
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }

  ///修改设备名称
  Future<bool> updateDeviceName(String deviceId, String deviceName) async {
    try {
      await methodChannel.invokeMethod(
          "updateDeviceName", {"deviceId": deviceId, "deviceName": deviceName});
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }

  ///删除设备
  Future<bool> deleteDevices(String deviceId) async {
    try {
      await methodChannel.invokeMethod("deleteDevices", {"deviceId": deviceId});
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }

  ///获得开锁记录
  Future<dynamic> lockRecord(String deviceId, int page) async {
    try {
      var str = await methodChannel
          .invokeMethod("lockRecord", {"deviceId": deviceId, "page": page});
      var decode = convert.json.decode(str);
      return decode["data"] == null ? [] : convert.json.decode(decode["data"]);
    } catch (e) {
      rethrow;
    }
  }

  ///监听设备信息发生变化
  StreamSubscription addDeviceChangeObserver(
      void Function(dynamic data) event) {
    StreamSubscription streamSubscription = _deviceChangeEvent.listen(event);
    _subscriptionList.add(streamSubscription);
    return streamSubscription;
  }

  ///取消监听设备信息发生变化
  ///当参数subscription不为空代表取消指定监听，为空代表取消所有监听
  removeDeviceChangeObserver({StreamSubscription? subscription}) {
    if (subscription != null) {
      subscription.cancel();
      _subscriptionList.remove(subscription);
    } else {
      for (var value in _subscriptionList) {
        value.cancel();
      }
      _subscriptionList.clear();
    }
  }

  ///绑定网关

  Future<bool> bindRelay(String superDeviceId, String deviceId) async {
    try {
      await methodChannel.invokeMethod(
          "bindRelay", {"superDeviceId": superDeviceId, "deviceId": deviceId});
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }

  /// 解绑绑定网关
  Future<bool> unbindRelay(String deviceId) async {
    try {
      await methodChannel.invokeMethod("unbindRelay", {"deviceId": deviceId});
      return Future.value(true);
    } catch (e) {
      rethrow;
    }
  }
}
